@@ -6,20 +6,22 @@ Subliminal uses `click <http://click.pocoo.org>`_ to provide a powerful :abbr:`C
 from __future__ import division
 from collections import defaultdict
 from datetime import timedelta
+import glob
 import json
 import logging
 import os
 import re
 
+from appdirs import AppDirs
 from babelfish import Error as BabelfishError, Language
 import click
 from dogpile.cache.backends.file import AbstractFileLock
-from dogpile.core import ReadWriteMutex
+from dogpile.util.readwrite_lock import ReadWriteMutex
 from six.moves import configparser
 
-from subliminal import (Episode, Movie, ProviderPool, Video, __version__, check_video, provider_manager, region,
-                        save_subtitles, scan_video, scan_videos)
-from subliminal.subtitle import compute_score
+from subliminal import (AsyncProviderPool, Episode, Movie, Video, __version__, check_video, compute_score, get_scores,
+                        provider_manager, refine, refiner_manager, region, save_subtitles, scan_video, scan_videos)
+from subliminal.core import ARCHIVE_EXTENSIONS, search_external_subtitles
 
 logger = logging.getLogger(__name__)
 
@@ -45,7 +47,7 @@ class MutexLock(AbstractFileLock):
 
 
 class Config(object):
-    """A :class:`~configparser.SafeConfigParser` wrapper to store configuration.
+    """A :class:`~configparser.ConfigParser` wrapper to store configuration.
 
     Interaction with the configuration is done with the properties.
 
@@ -61,6 +63,7 @@ class Config(object):
         self.config.add_section('general')
         self.config.set('general', 'languages', json.dumps(['en']))
         self.config.set('general', 'providers', json.dumps(sorted([p.name for p in provider_manager])))
+        self.config.set('general', 'refiners', json.dumps(sorted([r.name for r in refiner_manager])))
         self.config.set('general', 'single', str(0))
         self.config.set('general', 'embedded_subtitles', str(1))
         self.config.set('general', 'age', str(int(timedelta(weeks=2).total_seconds())))
@@ -92,6 +95,14 @@ class Config(object):
     def providers(self, value):
         self.config.set('general', 'providers', json.dumps(sorted([p.lower() for p in value])))
 
+    @property
+    def refiners(self):
+        return json.loads(self.config.get('general', 'refiners'))
+
+    @refiners.setter
+    def refiners(self, value):
+        self.config.set('general', 'refiners', json.dumps([r.lower() for r in value]))
+
     @property
     def single(self):
         return self.config.getboolean('general', 'single')
@@ -195,7 +206,9 @@ AGE = AgeParamType()
 
 PROVIDER = click.Choice(sorted(provider_manager.names()))
 
-app_dir = click.get_app_dir('subliminal')
+REFINER = click.Choice(sorted(refiner_manager.names()))
+
+dirs = AppDirs('subliminal')
 cache_file = 'subliminal.dbm'
 config_file = 'config.ini'
 
@@ -203,15 +216,16 @@ config_file = 'config.ini'
 @click.group(context_settings={'max_content_width': 100}, epilog='Suggestions and bug reports are greatly appreciated: '
              'https://github.com/Diaoul/subliminal/')
 @click.option('--addic7ed', type=click.STRING, nargs=2, metavar='USERNAME PASSWORD', help='Addic7ed configuration.')
+@click.option('--legendastv', type=click.STRING, nargs=2, metavar='USERNAME PASSWORD', help='LegendasTV configuration.')
 @click.option('--opensubtitles', type=click.STRING, nargs=2, metavar='USERNAME PASSWORD',
               help='OpenSubtitles configuration.')
 @click.option('--subscenter', type=click.STRING, nargs=2, metavar='USERNAME PASSWORD', help='SubsCenter configuration.')
-@click.option('--cache-dir', type=click.Path(writable=True, resolve_path=True, file_okay=False), default=app_dir,
+@click.option('--cache-dir', type=click.Path(writable=True, file_okay=False), default=dirs.user_cache_dir,
               show_default=True, expose_value=True, help='Path to the cache directory.')
 @click.option('--debug', is_flag=True, help='Print useful information for debugging subliminal and for reporting bugs.')
 @click.version_option(__version__)
 @click.pass_context
-def subliminal(ctx, addic7ed, opensubtitles, subscenter, cache_dir, debug):
+def subliminal(ctx, addic7ed, legendastv, opensubtitles, subscenter, cache_dir, debug):
     """Subtitles, faster than your thoughts."""
     # create cache directory
     try:
@@ -235,6 +249,8 @@ def subliminal(ctx, addic7ed, opensubtitles, subscenter, cache_dir, debug):
     ctx.obj = {'provider_configs': {}}
     if addic7ed:
         ctx.obj['provider_configs']['addic7ed'] = {'username': addic7ed[0], 'password': addic7ed[1]}
+    if legendastv:
+        ctx.obj['provider_configs']['legendastv'] = {'username': legendastv[0], 'password': legendastv[1]}
     if opensubtitles:
         ctx.obj['provider_configs']['opensubtitles'] = {'username': opensubtitles[0], 'password': opensubtitles[1]}
     if subscenter:
@@ -248,7 +264,8 @@ def subliminal(ctx, addic7ed, opensubtitles, subscenter, cache_dir, debug):
 def cache(ctx, clear_subliminal):
     """Cache management."""
     if clear_subliminal:
-        os.remove(os.path.join(ctx.parent.params['cache_dir'], cache_file))
+        for file in glob.glob(os.path.join(ctx.parent.params['cache_dir'], cache_file) + '*'):
+            os.remove(file)
         click.echo('Subliminal\'s cache cleared.')
     else:
         click.echo('Nothing done.')
@@ -258,6 +275,7 @@ def cache(ctx, clear_subliminal):
 @click.option('-l', '--language', type=LANGUAGE, required=True, multiple=True, help='Language as IETF code, '
               'e.g. en, pt-BR (can be used multiple times).')
 @click.option('-p', '--provider', type=PROVIDER, multiple=True, help='Provider to use (can be used multiple times).')
+@click.option('-r', '--refiner', type=REFINER, multiple=True, help='Refiner to use (can be used multiple times).')
 @click.option('-a', '--age', type=AGE, help='Filter videos newer than AGE, e.g. 12h, 1w2d.')
 @click.option('-d', '--directory', type=click.STRING, metavar='DIR', help='Directory where to save subtitles, '
               'default is next to the video file.')
@@ -269,11 +287,14 @@ def cache(ctx, clear_subliminal):
 @click.option('-hi', '--hearing-impaired', is_flag=True, default=False, help='Prefer hearing impaired subtitles.')
 @click.option('-m', '--min-score', type=click.IntRange(0, 100), default=0, help='Minimum score for a subtitle '
               'to be downloaded (0 to 100).')
+@click.option('-w', '--max-workers', type=click.IntRange(1, 50), default=None, help='Maximum number of threads to use.')
+@click.option('-z/-Z', '--archives/--no-archives', default=True, show_default=True, help='Scan archives for videos '
+              '(supported extensions: %s).' % ', '.join(ARCHIVE_EXTENSIONS))
 @click.option('-v', '--verbose', count=True, help='Increase verbosity.')
 @click.argument('path', type=click.Path(), required=True, nargs=-1)
 @click.pass_obj
-def download(obj, provider, language, age, directory, encoding, single, force, hearing_impaired, min_score, verbose,
-             path):
+def download(obj, provider, refiner, language, age, directory, encoding, single, force, hearing_impaired, min_score,
+             max_workers, archives, verbose, path):
     """Download best subtitles.
 
     PATH can be an directory containing videos, a video file path or a video file name. It can be used multiple times.
@@ -301,20 +322,26 @@ def download(obj, provider, language, age, directory, encoding, single, force, h
                     logger.exception('Unexpected error while collecting non-existing path %s', p)
                     errored_paths.append(p)
                     continue
+                if not force:
+                    video.subtitle_languages |= set(search_external_subtitles(video.name, directory=directory).values())
+                refine(video, episode_refiners=refiner, movie_refiners=refiner, embedded_subtitles=not force)
                 videos.append(video)
                 continue
 
             # directories
             if os.path.isdir(p):
                 try:
-                    scanned_videos = scan_videos(p, subtitles=not force, embedded_subtitles=not force,
-                                                 subtitles_dir=directory)
+                    scanned_videos = scan_videos(p, age=age, archives=archives)
                 except:
                     logger.exception('Unexpected error while collecting directory path %s', p)
                     errored_paths.append(p)
                     continue
                 for video in scanned_videos:
+                    if not force:
+                        video.subtitle_languages |= set(search_external_subtitles(video.name,
+                                                                                  directory=directory).values())
                     if check_video(video, languages=language, age=age, undefined=single):
+                        refine(video, episode_refiners=refiner, movie_refiners=refiner, embedded_subtitles=not force)
                         videos.append(video)
                     else:
                         ignored_videos.append(video)
@@ -322,12 +349,15 @@ def download(obj, provider, language, age, directory, encoding, single, force, h
 
             # other inputs
             try:
-                video = scan_video(p, subtitles=not force, embedded_subtitles=not force, subtitles_dir=directory)
+                video = scan_video(p)
             except:
                 logger.exception('Unexpected error while collecting path %s', p)
                 errored_paths.append(p)
                 continue
+            if not force:
+                video.subtitle_languages |= set(search_external_subtitles(video.name, directory=directory).values())
             if check_video(video, languages=language, age=age, undefined=single):
+                refine(video, episode_refiners=refiner, movie_refiners=refiner, embedded_subtitles=not force)
                 videos.append(video)
             else:
                 ignored_videos.append(video)
@@ -363,15 +393,20 @@ def download(obj, provider, language, age, directory, encoding, single, force, h
 
     # download best subtitles
     downloaded_subtitles = defaultdict(list)
-    with ProviderPool(providers=provider, provider_configs=obj['provider_configs']) as pool:
+    with AsyncProviderPool(max_workers=max_workers, providers=provider, provider_configs=obj['provider_configs']) as p:
         with click.progressbar(videos, label='Downloading subtitles',
                                item_show_func=lambda v: os.path.split(v.name)[1] if v is not None else '') as bar:
             for v in bar:
-                subtitles = pool.download_best_subtitles(pool.list_subtitles(v, language - v.subtitle_languages),
-                                                         v, language, min_score=v.scores['hash'] * min_score / 100,
-                                                         hearing_impaired=hearing_impaired, only_one=single)
+                scores = get_scores(v)
+                subtitles = p.download_best_subtitles(p.list_subtitles(v, language - v.subtitle_languages),
+                                                      v, language, min_score=scores['hash'] * min_score / 100,
+                                                      hearing_impaired=hearing_impaired, only_one=single)
                 downloaded_subtitles[v] = subtitles
 
+        if p.discarded_providers:
+            click.secho('Some providers have been discarded due to unexpected errors: %s' %
+                        ', '.join(p.discarded_providers), fg='yellow')
+
     # save subtitles
     total_subtitles = 0
     for v, subtitles in downloaded_subtitles.items():
@@ -385,23 +420,23 @@ def download(obj, provider, language, age, directory, encoding, single, force, h
 
         if verbose > 1:
             for s in saved_subtitles:
-                matches = s.get_matches(v, hearing_impaired=hearing_impaired)
-                score = compute_score(matches, v)
+                matches = s.get_matches(v)
+                score = compute_score(s, v)
 
                 # score color
                 score_color = None
+                scores = get_scores(v)
                 if isinstance(v, Movie):
-                    if score < v.scores['title']:
+                    if score < scores['title']:
                         score_color = 'red'
-                    elif score < v.scores['title'] + v.scores['year'] + v.scores['release_group']:
+                    elif score < scores['title'] + scores['year'] + scores['release_group']:
                         score_color = 'yellow'
                     else:
                         score_color = 'green'
                 elif isinstance(v, Episode):
-                    if score < v.scores['series'] + v.scores['season'] + v.scores['episode']:
+                    if score < scores['series'] + scores['season'] + scores['episode']:
                         score_color = 'red'
-                    elif score < (v.scores['series'] + v.scores['season'] + v.scores['episode'] +
-                                  v.scores['release_group']):
+                    elif score < scores['series'] + scores['season'] + scores['episode'] + scores['release_group']:
                         score_color = 'yellow'
                     else:
                         score_color = 'green'
@@ -409,16 +444,16 @@ def download(obj, provider, language, age, directory, encoding, single, force, h
                 # scale score from 0 to 100 taking out preferences
                 scaled_score = score
                 if s.hearing_impaired == hearing_impaired:
-                    scaled_score -= v.scores['hearing_impaired']
-                scaled_score *= 100 / v.scores['hash']
+                    scaled_score -= scores['hearing_impaired']
+                scaled_score *= 100 / scores['hash']
 
                 # echo some nice colored output
                 click.echo('  - [{score}] {language} subtitle from {provider_name} (match on {matches})'.format(
-                    score=click.style('{:5.1f}'.format(scaled_score), fg=score_color, bold=score >= v.scores['hash']),
+                    score=click.style('{:5.1f}'.format(scaled_score), fg=score_color, bold=score >= scores['hash']),
                     language=s.language.name if s.language.country is None else '%s (%s)' % (s.language.name,
                                                                                              s.language.country.name),
                     provider_name=s.provider_name,
-                    matches=', '.join(sorted(matches, key=v.scores.get, reverse=True))
+                    matches=', '.join(sorted(matches, key=scores.get, reverse=True))
                 ))
 
     if verbose == 0:
